# -*- coding: utf-8 -*-
##############################################################################
#
#   Copyright (c) 2011 Camptocamp SA (http://www.camptocamp.com)
#   @author Nicolas Bessi, Joel Grand-Guillaume, Alexandre Fayolle
#
# WARNING: This program as such is intended to be used by professional
# programmers who take the whole responsability of assessing all potential
# consequences resulting from its eventual inadequacies and bugs
# End users who are looking for a ready-to-use solution with commercial
# garantees and support are strongly adviced to contract a Free Software
# Service Company
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
## -*- coding: utf-8 -*-

import operator
from report import report_sxw
import pooler
import logging
from os.path import commonprefix

_logger = logging.getLogger(__name__)

class NullMove(object):
    """helper class to generate empty lines in the delivery report"""
    def __init__(self):
        self.product_id = NullObj()
        self.dispatch_id = NullObj()
        self.product_qty = ''

class NullObj(object):
    """the null obj has any attribute you want with an empty string as the value"""
    def __getattr__(self, attr):
        return ''


class DispatchAgregation(object):
    """group moves from a single dispatch by source and dest locations"""
    def __init__(self, dispatch, moves_by_loc):
        self.dispatch_id = dispatch
        self.moves_by_loc = moves_by_loc

    @property
    def picker_id(self):
        return self.dispatch_id.picker_id

    @property
    def dispatch_name(self):
        return self.dispatch_id.name


    def exists(self):
        return False

    def __hash__(self):
        return hash(self.dispatch_id.id)

    def __eq__(self, other):
        return (self.dispatch_id.id == other.dispatch_id.id)

    def iter_locations(self):
        for locations in self.moves_by_loc:
            offset = commonprefix(locations).rfind('/') + 1
            display_locations = tuple(loc[offset:].strip() for loc in locations)
            yield display_locations, self._product_quantity(locations)
    def _product_quantity(self, locations):
        """iterate over the different products concerned by the moves for the specified locations
        with their total quantity, sorted by product default_code

        locations: a tuple (source_location, dest_location)
        """
        products = {}
        product_qty = {}
        carrier = {}
        moves = self.moves_by_loc[locations]
        _logger.debug('move ids %s',  moves)
        for move in moves:
            p_code = move.product_id.default_code
            products[p_code] = move.product_id
            carrier[p_code] = move.picking_id.carrier_id and move.picking_id.carrier_id.partner_id.name or ''
            if p_code not in product_qty:
                product_qty[p_code] = move.product_qty
            else:
                product_qty[p_code] += move.product_qty
        for p_code in sorted(products):
            yield products[p_code], product_qty[p_code], carrier[p_code]

class PrintDispatch(report_sxw.rml_parse):

    def __init__(self, cursor, uid, name, context):
        super(PrintDispatch, self).__init__(cursor, uid, name, context=context)
        self.pool = pooler.get_pool(self.cr.dbname)
        self.cursor = self.cr
        self.uid = uid
        self.numeration_type = False
        self.localcontext.update({})

    def _get_form_param(self, param, data, default=False):
        return data.get('form', {}).get(param, default) or default

    def set_context(self, objects, data, ids, report_type=None):
        #!! data form is manually set in wizard
        new_objects = []
        location_obj = self.pool.get('stock.location')
        for dispatch in objects:
            moves_by_loc = {}
            for move in dispatch.move_ids:
                if move.state == 'assigned':
                    key = location_obj.name_get(self.cursor, self.uid,
                                                [move.location_id.id,
                                                 move.location_dest_id.id])
                    key = tuple(name for _id, name in key)
                    moves_by_loc.setdefault(key, []).append(move)
            _logger.debug('agreg %s ', moves_by_loc)
            new_objects.append(DispatchAgregation(dispatch, moves_by_loc))
        return super(PrintDispatch, self).set_context(new_objects, data, ids, report_type=report_type)

report_sxw.report_sxw('report.webkit.dispatch_order',
                      'picking.dispatch',
                      'addons/picking_dispatch/report/dispatch.html.mako',
                      parser=PrintDispatch)
